var bcrypt = require('bcrypt-nodejs');

/* The UsersDAO to be constructed with a connected database object */
function UsersDAO(db) {
    "use strict";

    if (false === (this instanceof UsersDAO)) {
        console.log('UsersDAO constructor called without "new" operator');
        return new UsersDAO(db);
    }

    this.addUser = function(username, password, email, callback) {
        "use strict";

        // Generate password hash
        var salt = bcrypt.genSaltSync();
        var password_hash = bcrypt.hashSync(password, salt);

	var user = {'email': email, 'password': password_hash, 'username': username,'type':'user'};
  	db.save('ID_'+Math.random(), user, function (err, res) {
      	      if (err) {
		  // Handle error
		  res += ' SAVE ERROR: Could not save record!!\n';
	      } else {
		  // Handle success
		  res += ' SUCESSFUL SAVE\n';
	      }
	      return callback(err, null);
	   });

    }

    this.validateLogin = function(username, password, callback) {
        "use strict";

	db.view('user/byUsername', { key: username }, function (err, doc) {
	    if (err) return callback(err, null);
            if (doc) {

                if (bcrypt.compareSync(password, doc[0].value.password)) {
		    console.log('here we are');	
                    callback(null, doc);
                }
                else {
                    var invalid_password_error = new Error("Invalid password");
                    // Set an extra field for any error which is not a db error
                    invalid_password_error.invalid_password = true;
                    callback(invalid_password_error, null);
                }
            }
            else {
                var no_such_user_error = new Error("User: " + user + " does not exist");
                no_such_user_error.no_such_user = true;
                callback(no_such_user_error, null);
            }

  	});

    }
}

module.exports.UsersDAO = UsersDAO;
